/*  BotikBalance - User's account balance informer for Botik Network 
 *                 for Windows 9x/ME/NT/2000/XP/2003
 *  Copyright (C) 2005 Alexander V. Melnikoff
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public Licensse as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

/* Sorry for the long lines in the sources, I have wide screen... ;-)
 */

#include <stdafx.h>
#include <shellapi.h>
#include <notifyicon.h>

CNotifyIcon::CNotifyIcon(void)
{
	cfdprintf(_T("CNotifyIcon::CNotifyIcon"));

	m_hWnd     = NULL;
	m_uID      = NULL;
	m_hInst    = NULL;
	m_nShowPos = -1; // Means not show...

	m_nFrom    = -1;
	m_nTo      = -1;

	cfdprintf(_T("!CNotifyIcon::CNotifyIcon"));
};

CNotifyIcon::~CNotifyIcon(void)
{
	cfdprintf(_T("CNotifyIcon::~CNotifyIcon"));
	cfdprintf(_T("!CNotifyIcon::~CNotifyIcon"));
};

BOOL CNotifyIcon::Create(HINSTANCE hInst_, HWND hWnd_, UINT uID_)
{
	cfdprintf(_T("CNotifyIcon::Create"));

	assert(m_hWnd == NULL);
	assert(m_uID == NULL);
	assert(m_hInst == NULL);

	m_hWnd  = hWnd_;
	m_uID   = uID_;
	m_hInst = hInst_;

	cfdprintf(_T("!CNotifyIcon::Create"));

	return(TRUE);
};

BOOL CNotifyIcon::Destroy()
{
	cfdprintf(_T("CNotifyIcon::Destroy"));

	for(UI2HIIT it = m_mapUint2Hico.begin(); it != m_mapUint2Hico.end(); it++)
	{
		cfdprintf(_T("CNotifyIcon::Destroy: Freeing HICON"));
		DestroyIcon(*it);
	};

	m_mapUint2Hico.clear();
	m_mapUint2Str.clear();

	cfdprintf(_T("!CNotifyIcon::Destroy"));

	return(TRUE);
};

BOOL CNotifyIcon::AddIcon(UINT uIcon_, LPCTSTR szFormat_, ...)
{
	HICON	hIcon;
	BOOL	fOk = FALSE;
	LPTSTR  szBuf;
	INT     nLen;
	va_list pArgs;

	cfdprintf(_T("CNotifyIcon::AddIcon"));

//	m_mapUint2Str.resize(m_mapUint2Str.size()+1);
//	m_mapUint2Hico.resize(m_mapUint2Hico.size()+1);

	hIcon = (HICON)LoadImage(m_hInst, MAKEINTRESOURCE(uIcon_), IMAGE_ICON, 16, 16, 0);
	if(hIcon != NULL)
	{
		va_start(pArgs, szFormat_);
		nLen = _vsctprintf(szFormat_, pArgs);
		szBuf = new TCHAR[nLen + 1];
		_vstprintf(szBuf, szFormat_, pArgs);
		va_end(pArgs);

		m_mapUint2Str.push_back(szBuf);
		m_mapUint2Hico.push_back(hIcon);

		delete[] szBuf;

		fOk = TRUE;
	};

	cfdprintf(_T("!CNotifyIcon::AddIcon: m_mapUint2Str: %d; m_mapUint2Hico: %d; hIcon: %d"), m_mapUint2Str.size(), m_mapUint2Hico.size(), hIcon);

	return(fOk);
};

BOOL CNotifyIcon::ChangeText(UINT uPos_, LPCTSTR szFormat_, ...)
{
	BOOL	fOk = FALSE;
	LPTSTR  szBuf;
	INT     nLen;
	va_list pArgs;

	cfdprintf(_T("CNotifyIcon::ChangeText"));

	if(uPos_ >= 0 && m_mapUint2Str.size() > uPos_)
	{
		va_start(pArgs, szFormat_);
		nLen = _vsctprintf(szFormat_, pArgs);
		szBuf = new TCHAR[nLen + 1];
		_vstprintf(szBuf, szFormat_, pArgs);
		va_end(pArgs);

		m_mapUint2Str[uPos_] = szBuf;

		delete[] szBuf;

		if(uPos_ == m_nShowPos) // To update currently showing icon
			ShowIcon(uPos_);
	};

	return(fOk);
};

BOOL CNotifyIcon::ShowIcon(UINT uPos_)
{
	NOTIFYICONDATA	nid;
	BOOL fOk = FALSE;

//	cfdprintf(_T("CNotifyIcon::ShowIcon"));

	assert(uPos_ >= 0);

	if(uPos_ >= 0 && m_mapUint2Str.size() > uPos_)
	{
		nid.cbSize = sizeof(NOTIFYICONDATA);
		nid.hWnd   = m_hWnd;
		nid.uID    = m_uID;
		nid.uFlags = NIF_ICON|NIF_MESSAGE|NIF_TIP;
		nid.hIcon  = m_mapUint2Hico.at(uPos_);
		nid.uCallbackMessage = WM_USER_NOTIFYICON;
		_tcscpy(nid.szTip, m_mapUint2Str.at(uPos_).c_str());

		fOk = Shell_NotifyIcon(m_nShowPos == -1 ? NIM_ADD : NIM_MODIFY, &nid);

//		cfdprintf(_T("!CNotifyIcon::ShowIcon: nid.hIcon: %d; nid.szTip: %s"), nid.hIcon, nid.szTip);

		m_nShowPos = (fOk ? uPos_ : m_nShowPos);
	};

//	cfdprintf(_T("!CNotifyIcon::ShowIcon: fOk: %d"), fOk);

	return(fOk);
};

BOOL CNotifyIcon::HideIcon()
{
	NOTIFYICONDATA	nid;

	if(m_nShowPos < 0) return(TRUE);

	nid.cbSize = sizeof(NOTIFYICONDATA);
	nid.hWnd   = m_hWnd;
	nid.uID    = m_uID;
	nid.uFlags = 0;

	m_nShowPos = -1;

	return(Shell_NotifyIcon(NIM_DELETE, &nid));
};

BOOL CNotifyIcon::SetRotatingRange(UINT uFrom_, UINT uTo_)
{
	m_nFrom = uFrom_;
	m_nTo = uTo_;

	return(TRUE);
};

LRESULT CNotifyIcon::OnTimer(HWND hWnd_, WPARAM wParam_, LPARAM lParam_)
{
	UINT nNextPos;

	if(m_nFrom < 0 || m_nTo < 0 || m_nShowPos < -1)
		return(FALSE);

	if(m_nShowPos < m_nFrom || m_nShowPos >= m_nTo)
		nNextPos = m_nFrom;
	else
		nNextPos = m_nShowPos + 1;

	ShowIcon(nNextPos);

	return(TRUE);
};

LRESULT CNotifyIcon::OnTaskbarCreated(HWND hWnd_, WPARAM wParam_, LPARAM lParam_)
{
	INT nPos = m_nShowPos;
	m_nShowPos = -1;
	ShowIcon(nPos);
	return(TRUE);
};