/*  BotikBalance - User's account balance informer for Botik Network 
 *                 for Windows 9x/ME/NT/2000/XP/2003
 *  Copyright (C) 2005 Alexander V. Melnikoff
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public Licensse as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

/* Sorry for the long lines in the sources, I have wide screen... ;-)
 */

#include <stdafx.h>
#include <scripthost.h>
#include <scriptenginesguids.h>
#include <botikbalancecom.h>

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CBotikBalanceCom::CBotikBalanceCom(void)
{
	cfdprintf("CBotikBalanceCom::CBotikBalanceCom");

	m_cRef       = 1;
	m_pITypeInfo = NULL;

	m_pSH   = NULL;
	m_pIAS  = NULL;
	m_pIASP = NULL;

	InterlockedIncrement(&g_lDllRefCnt);

	cfdprintf("!CBotikBalanceCom::CBotikBalanceCom");
};

CBotikBalanceCom::~CBotikBalanceCom(void)
{
	cfdprintf("!CBotikBalanceCom::~CBotikBalanceCom");

	// Stop machine!!!
	m_pIAS->SetScriptState(SCRIPTSTATE_DISCONNECTED);
	m_pIAS->SetScriptState(SCRIPTSTATE_CLOSED);
	UnInitScriptEngine();

	if(m_pITypeInfo) m_pITypeInfo->Release();

	InterlockedDecrement(&g_lDllRefCnt);

	cfdprintf("!CBotikBalanceCom::~CBotikBalanceCom");
};

//////////////////////////////////////////////////////////////////////
// IUnknown
//////////////////////////////////////////////////////////////////////

HRESULT __stdcall CBotikBalanceCom::QueryInterface(const IID& riid_, void** ppv_)
{
	if(riid_ == IID_IUnknown || riid_ == IID_IBotikBalance || riid_ == IID_IDispatch) 
	{
		*ppv_ = static_cast<IBotikBalance*>(this);
	} 
	else 
	{
		*ppv_ = NULL;
		return(E_NOINTERFACE);
	};

	reinterpret_cast<IUnknown*>(*ppv_)->AddRef ();

	return(S_OK);
};

ULONG __stdcall CBotikBalanceCom::AddRef(void) 
{
	return(InterlockedIncrement(&m_cRef));
};

ULONG __stdcall CBotikBalanceCom::Release (void) 
{
	if(InterlockedDecrement(&m_cRef) == 0)
	{
		delete this;
		return(0);
	};
	return(m_cRef);
};

//////////////////////////////////////////////////////////////////////
// IDispatch
//////////////////////////////////////////////////////////////////////

HRESULT __stdcall CBotikBalanceCom::GetTypeInfoCount(UINT* pCountTypeInfo_) 
{
	*pCountTypeInfo_ = 1;

	return S_OK;
};

HRESULT __stdcall CBotikBalanceCom::GetTypeInfo(UINT iTypeInfo_, LCID, ITypeInfo** ppITypeInfo_) 
{
	*ppITypeInfo_ = NULL;
	if(iTypeInfo_ != 0) 
	{
		return DISP_E_BADINDEX;
	};

	// Call AddRef () and return pointer.
	m_pITypeInfo->AddRef();
	*ppITypeInfo_ = m_pITypeInfo;

	return S_OK;
};

/* It doesn't support locale. */
HRESULT __stdcall CBotikBalanceCom::GetIDsOfNames(const IID& riid_, OLECHAR** arrayNames_, 
												  UINT cNames_, LCID, DISPID* arrayDispIDs_) 
{
	if(riid_ != IID_NULL) 
	{
		return DISP_E_UNKNOWNINTERFACE;
	};

	return m_pITypeInfo->GetIDsOfNames(arrayNames_, cNames_, arrayDispIDs_);
};

/* It doesn't support locale. */
HRESULT __stdcall CBotikBalanceCom::Invoke(DISPID dispidMember_, const IID& riid_, 
										   LCID, WORD wFlags_, DISPPARAMS* pDispParams_, 
										   VARIANT* pvarRes_, EXCEPINFO* pEI_, UINT* pArgErr_) 
{
	if(riid_ != IID_NULL) 
	{
		return DISP_E_UNKNOWNINTERFACE;
	};
	::SetErrorInfo(0, NULL);

	return m_pITypeInfo->Invoke(static_cast<IDispatch*>(this), dispidMember_, 
								wFlags_, pDispParams_, pvarRes_, pEI_, pArgErr_);
};

//////////////////////////////////////////////////////////////////////
// IBotikBalance
//////////////////////////////////////////////////////////////////////

HRESULT __stdcall CBotikBalanceCom::UseRegistryIdentity(LONG lPlace_, BOOL* pfRet_)
{
	*pfRet_ = false;
	return(S_OK);
};

HRESULT __stdcall CBotikBalanceCom::GetData(BSTR bsName_, BSTR* pbsVal_)
{
	LPTSTR szVal;

	szVal = g_pDataCont->GetData(CW2T(bsName_));

	cfdprintf(_T("CBotikBalanceCom::GetData: Value: %s"), szVal);

	if(szVal)
	{
		*pbsVal_ = T2BSTR(szVal);
		delete[] szVal;
	};

	return(S_OK);
};

HRESULT __stdcall CBotikBalanceCom::Connect(LONG* plRet_)
{
	*plRet_ = m_pSH->GetBotikBalanceObj()->Fire_OnConnect();

	return(S_OK);
};

HRESULT __stdcall CBotikBalanceCom::Update(LONG* plRet_)
{
	*plRet_ = m_pSH->GetBotikBalanceObj()->Fire_OnUpdate();

	return(S_OK);
};

HRESULT __stdcall CBotikBalanceCom::Disconnect(LONG* plRet_)
{
	*plRet_ = m_pSH->GetBotikBalanceObj()->Fire_OnDisconnect();

	return(S_OK);
};

HRESULT __stdcall CBotikBalanceCom::put_AccountName(BSTR bsVal_)
{
	cfdprintf(_T("CBotikBalanceCom::put_AccountName: %s"), CW2T(bsVal_));
	g_pDataCont->SetAccount(CW2T(bsVal_));
	return(S_OK);
};

HRESULT __stdcall CBotikBalanceCom::put_User(BSTR bsVal_)
{
	cfdprintf(_T("CBotikBalanceCom::put_User: %s"), CW2T(bsVal_));
	g_pDataCont->SetUser(CW2T(bsVal_));
	return(S_OK);
};

HRESULT __stdcall CBotikBalanceCom::put_Password(BSTR bsVal_)
{
	cfdprintf(_T("CBotikBalanceCom::put_Password: %s"), CW2T(bsVal_));
	g_pDataCont->SetPassword(CW2T(bsVal_));
	return(S_OK);
};

HRESULT __stdcall CBotikBalanceCom::put_UseProxy(BOOL fVal_)
{
	g_pDataCont->SetUseProxy(fVal_);
	return(S_OK);
};

HRESULT __stdcall CBotikBalanceCom::get_UseProxy(BOOL* pfVal_)
{
	*pfVal_ = g_pDataCont->GetUseProxy();
	return(S_OK);
};

HRESULT __stdcall CBotikBalanceCom::put_ProxyServer(BSTR bsVal_)
{
	g_pDataCont->SetProxyServer(CW2T(bsVal_));
	return(S_OK);
};

HRESULT __stdcall CBotikBalanceCom::get_ProxyServer(BSTR* pbsVal_)
{
	*pbsVal_ = T2BSTR(g_pDataCont->GetProxyServer());
	return(S_OK);
};

HRESULT __stdcall CBotikBalanceCom::GetErrorDescription(LONG lCode_, [out,retval] BSTR* pbsRet_)
{
	switch(lCode_)
	{
	case BBO_RET_OK:
		*pbsRet_ = ::SysAllocString(L"  ");
		break;

	case BBO_RET_FAIL:
		*pbsRet_ = ::SysAllocString(L"  ");
		break;

	case BBO_RET_INETOPEN:
		*pbsRet_ = ::SysAllocString(L"    ");
		break;

	case BBO_RET_INETCONN:
		*pbsRet_ = ::SysAllocString(L"    ");
		break;

	case BBO_RET_OPENREQ:
		*pbsRet_ = ::SysAllocString(L"   HTTP ");
		break;

	case BBO_RET_SENDREQ:
		*pbsRet_ = ::SysAllocString(L"   HTTP ");
		break;

	case BBO_RET_INETREAD:
		*pbsRet_ = ::SysAllocString(L"    ");
		break;

	case BBO_RET_WRONGMETHOD:
		*pbsRet_ = ::SysAllocString(L" HTTP ");
		break;

	case BBO_RET_WRONGPROXY:
		*pbsRet_ = ::SysAllocString(L" -  ");
		break;

	case BBO_RET_PARSEFAILED:
		*pbsRet_ = ::SysAllocString(L"    ,   Nadmin  ");
		break;

	case BBO_RET_LOGIN_BADPSWD:
		*pbsRet_ = ::SysAllocString(L"   Nadmin  ,    ");
		break;

	case BBO_RET_LOGIN_BADUSER:
		*pbsRet_ = ::SysAllocString(L"   Nadmin  ,    ");
		break;

	case BBO_RET_LOGIN_BADACCOUNT:
		*pbsRet_ = ::SysAllocString(L"   Nadmin  ,     ");
		break;

	case BBO_RET_LOGIN_NOTLOGGEDIN:
		*pbsRet_ = ::SysAllocString(L"   Nadmin  ,  ");
		break;

    case BBO_RET_LOGIN_BADPARAMS:
        *pbsRet_ = ::SysAllocString(L"   Nadmin  ,        LOGIN, PASSWORD  ORG");
        break;

	default:
		*pbsRet_ = ::SysAllocString(L"  ");
		break;
	};

	return(S_OK);
};

//////////////////////////////////////////////////////////////////////
// CBotikBalanceCom
//////////////////////////////////////////////////////////////////////

BOOL CBotikBalanceCom::Init(VOID)
{
	USES_CONVERSION;

	BOOL	fOk = FALSE;

	cfdprintf(_T("CBotikBalanceCom::Init"));

	__try
	{
		TCHAR	szPath[MAX_PATH+2+1]; // 2 is for resource id and slash, see below...
		HRESULT	hr;

		// At this step we need to load type library. The type library
		// for this object is located in this DLL resource section.
		// To load type library from this DLL, we need to know where 
		// this DLL is located. We need to load first type library here.
		GetModuleFileName((HMODULE)g_hModule, szPath, MAX_PATH);

		cfdprintf(_T("CBotikBalanceCom::Init: Loading type library from %s"), szPath);

		ITypeLib* pITypeLib = NULL;
		hr = ::LoadTypeLib(T2W(szPath), &pITypeLib);

		if(FAILED(hr)) 
		{
			cfdprintf(_T("CBotikBalanceCom::Init: Load type library failed (0x%x)"), hr);
			__leave;
		};

		// Get type information for interface object.
		hr = pITypeLib->GetTypeInfoOfGuid(IID_IBotikBalance, &m_pITypeInfo);

		pITypeLib->Release();
		if(FAILED(hr)) 
		{
			cfdprintf(_T("CBotikBalanceCom::Init: Get type information for interface object failed (0x%x)"), hr);
			__leave;
		};

		g_pDataCont = CDataCont::CreateObject();
		if(g_pDataCont == NULL)
		{
			cfdprintf(_T("CBotikBalanceCom::Init: Create CConfig object failed]"));
			__leave;
		};

		fOk = TRUE;

		cfdprintf(_T("CBotikBalanceCom::Init: Success"));
	}

	__finally
	{
		cfdprintf(_T("CBotikBalanceCom::Init: __finaly"));
	};

	cfdprintf(_T("!CBotikBalanceCom::Init"));

	return(fOk);
};

BOOL CBotikBalanceCom::InitScriptEngine(VOID)
{
	BOOL				fOk   = FALSE;
	HRESULT				hr;

	cfdprintf(_T("CBotikBalanceCom::InitScriptEngine"));

	__try
	{
		// Create script host object.
		m_pSH = CScriptHost::CreateObject();
		if(!m_pSH)
		{
			cfdprintf(_T("CBotikBalanceCom::InitScriptEngine: Create CScriptHost failed."));
			__leave;
		};

		if(!m_pSH->Init())
		{
			cfdprintf(_T("CBotikBalanceCom::InitScriptEngine: Create CScriptHost::Init failed."));
			__leave;
		};

		// Create an instance of JScript engine.
		hr = ::CoCreateInstance(CLSID_JScript, NULL, CLSCTX_INPROC_SERVER, 
								IID_IActiveScript, (void**)&m_pIAS);
		if(FAILED(hr))
		{
			cfdprintf(_T("CBotikBalanceCom::InitScriptEngine: Create an instance of JScript engine failed."));
			__leave;
		};

		// Get an IActiveScriptParse interface.
		hr = m_pIAS->QueryInterface(IID_IActiveScriptParse, (void**)&m_pIASP);
		if(FAILED(hr))
		{
			cfdprintf(_T("CBotikBalanceCom::InitScriptEngine: Get an IActiveScriptParse interface failed."));
			__leave;
		};

		// Here we pass our script host to the ActiveScript.
		hr = m_pIAS->SetScriptSite((IActiveScriptSite*)m_pSH);
		if(FAILED(hr))
		{
			cfdprintf(_T("CBotikBalanceCom::InitScriptEngine: Set script site failed."));
			__leave;
		};

		// Well, let's initialize the script engine.
		hr = m_pIASP->InitNew();
		if(FAILED(hr))
		{
			cfdprintf(_T("CBotikBalanceCom::InitScriptEngine: Script engine initialization failed."));
			__leave;
		};

		// Let's add our host object into script namespace.
		hr = m_pIAS->AddNamedItem(OLESTR("BotikBalance"), SCRIPTITEM_ISVISIBLE|SCRIPTITEM_ISSOURCE);
		if(FAILED(hr))
		{
			cfdprintf(_T("CBotikBalanceCom::InitScriptEngine: Add BotikBalance object into script namespace failed"));
			__leave;
		};

		fOk = TRUE;
	}

	__finally
	{
		if(!fOk)
			UnInitScriptEngine();

		cfdprintf(_T("CBotikBalanceCom::InitScriptEngine: __finally"));
	};

	cfdprintf(_T("!CBotikBalanceCom::InitScriptEngine"));

	return(fOk);
};

// XXX 
// Temporary hack. Solve it later...
// See a set of #undefs at the end of following method.
#define __finally	cleanup:
#define __try
#define __leave		goto cleanup
BOOL CBotikBalanceCom::RunBotikBalanceObjScript(VOID)
{
	HRESULT   hr;
	EXCEPINFO ei;
	BOOL      fOk      = FALSE;
	BSTR      bsCode = NULL;
	BSTR      bsItemName = NULL;
	HRSRC     hrc;
	HGLOBAL   hgl;
	DWORD     dwSize;
	LPTSTR    szScriptCode = NULL;

	cfdprintf(_T("CBotikBalanceCom::RunBotikBalanceObjScript"));

	__try
	{
		// Load JScript code from resources.
		hrc = ::FindResource((HMODULE)g_hModule, MAKEINTRESOURCE(IDR_BOTIK_BALANCE_OBJ_JS), RT_HTML);
		if(hrc == NULL) 
		{
			cfdprintf(_T("CBotikBalanceCom::RunBotikBalanceObjScript: FindResource failed"));
			__leave;
		};

		dwSize = ::SizeofResource((HMODULE)g_hModule, hrc);

		hgl = ::LoadResource((HMODULE)g_hModule, hrc);
		if(hgl == NULL) 
		{
			cfdprintf(_T("CBotikBalanceCom::RunBotikBalanceObjScript: LoadResource failed"));
			__leave;
		};

		szScriptCode = (LPTSTR)LockResource(hgl);
		if(szScriptCode == NULL) 
		{
			cfdprintf(_T("CBotikBalanceCom::RunBotikBalanceObjScript: LockResource failed"));
			__leave;
		};
		bsCode = A2WBSTR(szScriptCode, dwSize);
		FreeResource(hgl);

		// Parse script code.
		hr = m_pIASP->ParseScriptText(bsCode, NULL, NULL, NULL, 0, 0, 0, NULL, &ei);
		::SysFreeString(bsCode);
		if(FAILED(hr)) 
		{
			cfdprintf(_T("CBotikBalanceCom::RunBotikBalanceObjScript: m_pIASP->ParseScriptText failed"));
			__leave;
		};

		// Well, now we are ready to start your script code.
		hr = m_pIAS->SetScriptState(SCRIPTSTATE_CONNECTED);
		if(FAILED(hr)) 
		{
			cfdprintf(_T("CBotikBalanceCom::RunBotikBalanceObjScript: m_pIAS->SetScriptState failed"));
			__leave;
		};

		fOk = TRUE;

	}

	__finally
	{
		if(!fOk)
			MessageBox(NULL, _T("Oops... Internal programm error!!!"), BBOBJ_OBJ_NAME, MB_OK|MB_ICONERROR);
	};

	cfdprintf(_T("!CBotikBalanceCom::RunBotikBalanceObjScript"));

	return(fOk);
};
#undef __finally
#undef __try
#undef __leave

VOID CBotikBalanceCom::UnInitScriptEngine()
{
	cfdprintf(_T("CBotikBalanceCom::UnInitScriptEngine"));

	if(m_pIASP) 
	{
		m_pIAS->Close();
		m_pIASP->Release();
		m_pIASP = NULL;
	};

	if(m_pIAS)
	{
		m_pIAS->Release();
		m_pIAS = NULL;
	};

	if(m_pSH) 
	{
		delete m_pSH;
		m_pSH = NULL;
	};

	cfdprintf(_T("!CBotikBalanceCom::UnInitScriptEngine"));
};